/* ax88140.c: A ASIX AX88140 MAC chip ethernet driver for linux.
		(derived from tulip.c by Donald Becker)

	This software may be used and distributed according to the terms
	of the GNU Public License, incorporated herein by reference.

	The author may be reached as william@asix.com.tw. */

static const char *version ="ax88140.c:v0.01 10/24/97 william@asix.com.tw\n";

// A few user-configurable values.
#ifndef	AX88140_DEBUG
#define	AX88140_DEBUG	1
#endif

// Set if the PCI BIOS detects the chips on a multiport board backwards.
#ifdef REVERSE_PROBE_ORDER
static int reverse_probe =1;
#else
static int reverse_probe =0;
#endif

/* Keep the ring sizes a power of two for efficiency.
   Making the Tx ring too large decreases the effectiveness of channel
   bonding and packet priority.
   There are no ill effects from too-large receive rings. */

#define TX_RING_SIZE	16
#define RX_RING_SIZE	32

// Set the copy breakpoint for the copy-only-tiny-buffer Rx structure.
static const rx_copybreak =200;

// The following example shows how to always use the 10base2 port.
#ifdef notdef
#define AX88140_DEFAULT_MEDIA	1	// 1 == 10base2
#define AX88140_NO_MEDIA_SWITCH		// Don't switch from this port
#endif

// Define to force full-duplex operation.
//#define	AX88140_FULL_DUPLEX	1

// Operational parameters that usually are not changed.
// Time in jiffies before concluding the transmitter is hung.
#define TX_TIMEOUT  ((2000*HZ)/1000)

#include <linux/config.h>
#ifdef MODULE
#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif
#include <linux/module.h>
#include <linux/version.h>
#else
#define MOD_INC_USE_COUNT
#define MOD_DEC_USE_COUNT
#endif

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/timer.h>
#include <linux/ptrace.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/malloc.h>
#include <linux/interrupt.h>
#include <linux/pci.h>
#include <linux/bios32.h>
#include <asm/processor.h>	// Processor type for cache alignment.
#include <asm/bitops.h>
#include <asm/io.h>
#include <asm/dma.h>

#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>

/* Kernel compatibility defines, common to David Hind's PCMCIA package.
   This is only in the support-all-kernels source code. */
#include <linux/version.h>	// Evil, but neccessary

#if defined (LINUX_VERSION_CODE) && LINUX_VERSION_CODE < 0x10300
#define	RUN_AT(x)	(x)	// What to put in timer->expires.
#define	DEV_ALLOC_SKB(len)	alloc_skb(len, GFP_ATOMIC)
#define	virt_to_bus(addr)	((unsigned long)addr)
#define	bus_to_virt(addr)	((void*)addr)

#else	// 1.3.0 and later
#define	RUN_AT(x)	(jiffies +(x))
#define	DEV_ALLOC_SKB(len)	dev_alloc_skb(len +2)
#endif

#if defined (LINUX_VERSION_CODE) && LINUX_VERSION_CODE < 0x10338
#ifdef MODULE
#if !defined(CONFIG_MODVERSIONS) && !defined(__NO_VERSION__)
char kernel_version[] =UTS_RELEASE;
#endif
#else
#undef MOD_INC_USE_COUNT
#define MOD_INC_USE_COUNT
#undef MOD_DEC_USE_COUNT
#define MOD_DEC_USE_COUNT
#endif
#endif	// 1.3.38

#if (LINUX_VERSION_CODE >= 0x10344)
#define NEW_MULTICAST
#include <linux/delay.h>
#endif
#if (LINUX_VERSION_CODE >= 0x20100)
char kernel_version[] =UTS_RELEASE;
#endif
#ifdef SA_SHIRQ
#define IRQ(irq, dev_id, pt_regs) (irq, dev_id, pt_regs)
#else
#define IRQ(irq, dev_id, pt_regs) (irq, pt_regs)
#endif

#if (LINUX_VERSION_CODE < 0x20123)
#define test_and_set_bit(val, addr) set_bit(val, addr)
#endif

// This my implementation of shared IRQs, now only used for 1.2.13.
#ifdef HAVE_SHARED_IRQ
#define USE_SHARED_IRQ
#include <linux/shared_irq.h>
#endif

/* The total size is unusually large: The 21040 aligns each of its 16
   longword-wide registers on a quadword boundary. */
#define AX88140_TOTAL_SIZE 0x80

#ifdef HAVE_DEVLIST
struct netdev_entry ax88140_drv =
{"AX88140", ax88140_pci_probe, AX88140_TOTAL_SIZE, NULL};
#endif

// A few values that may be tweaked.
#define PKT_BUF_SZ	1536		// Size of each temporary Rx buffer.
#define	EEPROM_SIZE	128

/* This is a mysterious value that can be written to CSR11 in the 21040 (only)
   to support a pre-NWay full-duplex signaling mechanism using short frames.
   No one knows what it should be, but if left at its default value some
   10base2(!) packets trigger a full-duplex-request interrupt. */
#define FULL_DUPLEX_MAGIC	0x6969

#ifndef PCI_VENDOR_ID_ASIX		// Now defined in linux/pci.h
#define PCI_VENDOR_ID_ASIX		0x125B
#define PCI_DEVICE_ID_ASIX_AX88140	0x1400
#endif

#ifndef PCI_DEVICE_ID_DEC_AX88140_PLUS
#define PCI_DEVICE_ID_DEC_AX88140_PLUS	0x0014		// 21041.
#endif

#ifndef PCI_DEVICE_ID_DEC_AX88140_21142
#define PCI_DEVICE_ID_DEC_AX88140_21142	0x0019
#endif

// The rest of these values should never change.

static void ax88140_timer(unsigned long data);

// A table describing the chip types.
static struct ax88140_chip_table {
  int device_id;
  char *chip_name;
  int flags;
  void (*media_timer)(unsigned long data);
} ax88140_tbl[] ={
 {PCI_DEVICE_ID_ASIX_AX88140,"AX88140",0,ax88140_timer},
 {0, 0, 0, 0},
};

// This matches the table above.
// enum chips { DC21040=0, DC21041=1, DC21140=2, DC21142=3, };

static const char * const medianame[] ={
  "10baseT", "10base2", "AUI", "100baseTx",
  "10baseT-FD", "100baseTx-FD", "100baseT4", "100baseFx",
  "100baseFx-FD", "MII 10baseT", "MII 10baseT-FD", "MII",
  "10baseT(forced)", "MII 100baseTx", "MII 100baseTx-FD", "MII 100baseT4",
};

// A full-duplex map for above.
static const char media_fd[] =
{0,0,0,0,  0xff,0xff,0,0,  0xff,0,0xff,0x01, 0,0,0xff,0 };

/*
// 21041 transceiver register settings: 10-T, 10-2, AUI, 10-T, 10T-FD
static u16 t21041_csr13[] ={ 0xEF05, 0xEF09, 0xEF09, 0xEF01, 0xEF09, };
static u16 t21041_csr14[] ={ 0x7F3F, 0xF7FD, 0xF7FD, 0x7F3F, 0x7F3D, };
static u16 t21041_csr15[] ={ 0x0008, 0x0006, 0x000E, 0x0008, 0x0008, };

static u16 t21142_csr13[] ={ 0x0001, 0x0009, 0x0009, 0x0001, 0x0001, };
static u16 t21142_csr14[] ={ 0xFFFF, 0x0705, 0x0705, 0x7F3F, 0x7F3D, };
static u16 t21142_csr15[] ={ 0x0008, 0x0006, 0x000E, 0x0008, 0x0008, };
*/

// Offsets to the Command and Status Registers, "CSRs".  All accesses
// must be longword instructions and quadword aligned.
enum ax88140_offsets {
	CSR0=0,    CSR1=0x08, CSR2=0x10, CSR3=0x18, CSR4=0x20, CSR5=0x28,
	CSR6=0x30, CSR7=0x38, CSR8=0x40, CSR9=0x48, CSR10=0x50, CSR11=0x58,
	CSR12=0x60, CSR13=0x68, CSR14=0x70, CSR15=0x78 };

// The bits in the CSR5 status registers, mostly interrupt sources.
enum status_bits {
	TimerInt=0x800, TPLnkFail=0x1000, TPLnkPass=0x10,
	RxJabber=0x200, RxDied=0x100, RxNoBuf=0x80, RxIntr=0x40,
	TxFIFOUnderflow=0x20, TxJabber=0x08, TxNoBuf=0x04, TxDied=0x02,
	TxIntr=0x01,
};

// The Rx and Tx buffer descriptors.
struct ax88140_rx_desc {
	s32 status;
	s32 length;
	u32 buffer1, buffer2;
};

struct ax88140_tx_desc {
	s32 status;
	s32 length;
	u32 buffer1, buffer2;		// We use only buffer 1.
};

struct medialeaf {
	u8 type;
	u8 media;
	unsigned char *leafdata;
};

struct mediatable {
	u16 defaultmedia;
	u8 leafcount, csr12dir; 	// General purpose pin directions.
	unsigned has_mii:1;
	struct medialeaf mleaf[0];
};

struct mediainfo {
	struct mediainfo *next;
	int info_type;
	int index;
	struct non_mii {
		char media;
		unsigned char csr12val;
		char bitnum, flags;
	} non_mii;
	unsigned char *info;
};

struct ax88140_private {
	char devname[8];	// Used only for kernel debugging.
//	const char *product_name;
	struct device *next_module;
	struct ax88140_rx_desc rx_ring[RX_RING_SIZE];
	struct ax88140_tx_desc tx_ring[TX_RING_SIZE];

	// The saved address of a sent-in-place packet/buffer, for skfree().
	struct sk_buff* tx_skbuff[TX_RING_SIZE];

	// The addresses of receive-in-place skbuffs.
	struct sk_buff* rx_skbuff[RX_RING_SIZE];

//	char *rx_buffs; 	// Address of temporary Rx buffers.
	u32 multicast_bits[2];
	int chip_id;
	int revision;
#if LINUX_VERSION_CODE > 0x20139
	struct net_device_stats stats;
#else
	struct enet_statistics stats;
#endif
	struct timer_list timer;	//Media selection timer.
	unsigned int cur_rx, cur_tx;	//The next free ring entry
	unsigned int dirty_rx, dirty_tx;//The ring entries to be free()ed.
	unsigned int tx_full:1; 	// The Tx queue is full.
	unsigned int full_duplex:1;	// Full-duplex operation requested.
	unsigned int default_port:4;	// Last dev->if_port value.
	unsigned int media2:4;		// Secondary monitored media port.
	unsigned int medialock:1;	// Don't sense media type.
	unsigned int mediasense:1;	// Media sensing in progress.
	unsigned int csr6;		// Current CSR6 control settings.
	unsigned char sromdata[EEPROM_SIZE];	// Serial EEPROM contents.
	signed char phys[4];		// MII device addresses.
	struct mediatable *mtable;
	int cur_index;			// Current media index.
	unsigned char pci_bus, pci_device_fn;
//	int pad0, pad1; 		// Used for 8-byte alignment
};

// Used to pass the full-duplex flag, etc.
static int full_duplex[8] ={0, };
static int options[8] ={0, };


// Serial EEPROM section.
/* The main routine to parse the very complicated SROM structure.
   Search www.digital.com for "21X4 SROM" to get details.
   This code is very complex, and will require changes to support
   additional cards, so I'll be verbose about what is going on.	*/

// EEPROM_Ctrl bits.
#define EE_SHIFT_CLK	0x02	// EEPROM shift clock.
#define EE_CS		0x01	// EEPROM chip select.
#define EE_DATA_WRITE	0x04	// EEPROM chip data in.
#define EE_WRITE_0	0x01
#define EE_WRITE_1	0x05
#define EE_DATA_READ	0x08	// EEPROM chip data out.
#define EE_ENB		(0x2800 |EE_CS)

/* Delay between EEPROM clock transitions.
   The 1.2 code is a "nasty" timing loop, but PC compatible machines are
   *supposed* to delay an ISA-compatible period for the SLOW_DOWN_IO macro. */
#ifdef _LINUX_DELAY_H
#define srom_delay(nanosec)	udelay((nanosec +999)/1000)
#else
#define srom_delay(nanosec)	do {int _i =3; while (--_i >0) { __SLOW_DOWN_IO; }} while (0)
#endif

// The EEPROM commands include the alway-set leading bit.
#define EE_WRITE_CMD	(5 << 6)
#define EE_READ_CMD	(6 << 6)
#define EE_ERASE_CMD	(7 << 6)

static unsigned int read_srom(int ioaddr, int location)
{
	int i;
	unsigned short retval =0;
	int ee_addr =ioaddr +CSR9;
	int read_cmd =location |EE_READ_CMD;

	outl(EE_ENB & ~EE_CS, ee_addr);
	srom_delay(100);
	outl(EE_ENB, ee_addr);
	srom_delay(100);

	// Shift the read command bits out.
	for (i =10; i >= 0; i--) {
		short dataval =(read_cmd & (1 << i)) ? EE_DATA_WRITE :0;
		outl(EE_ENB |dataval, ee_addr);
		srom_delay(100);
		outl(EE_ENB |dataval |EE_SHIFT_CLK, ee_addr);
		srom_delay(150);
		outl(EE_ENB |dataval, ee_addr);	//Finish EEPROM a clock tick.
		srom_delay(250);
	}
	outl(EE_ENB, ee_addr);

	for (i =16; i > 0; i--) {	// read 16 bits
		outl(EE_ENB |EE_SHIFT_CLK, ee_addr);
		srom_delay(100);
		retval =(retval << 1) |((inl(ee_addr) &EE_DATA_READ) ? 1 :0);
		outl(EE_ENB, ee_addr);
		srom_delay(100);
	}

	// Terminate the EEPROM access.
	outl(EE_ENB & ~EE_CS, ee_addr);
	return retval;
}


/* Read and write the MII registers using software-generated serial
   MDIO protocol.  It is just different enough from the EEPROM protocol
   to not share code.  The maxium data clock rate is 2.5 Mhz. */
#define MDIO_SHIFT_CLK	0x10000
#define MDIO_DATA_WRITE0 0x00000
#define MDIO_DATA_WRITE1 0x20000
#define MDIO_ENB	0x00000	// Ignore the 0x02000 databook setting. 
#define MDIO_ENB_IN	0x40000
#define MDIO_DATA_READ	0x80000
#ifdef _LINUX_DELAY_H
#define mdio_delay()	udelay(1)
#else
#define mdio_delay()	__SLOW_DOWN_IO
#endif

static int mdio_read(int ioaddr, int phy_id, int location)
{
	int i;
	int read_cmd =(0xf6 << 10) |(phy_id << 5) |location;
	unsigned short retval =0;
	int mdio_addr =ioaddr +CSR9;

	// Establish sync by sending at least 32 logic ones.
	for (i =32; i >= 0; i--) {
		outl(MDIO_ENB |MDIO_DATA_WRITE1, mdio_addr);
		mdio_delay();
		outl(MDIO_ENB |MDIO_DATA_WRITE1 |MDIO_SHIFT_CLK, mdio_addr);
		mdio_delay();
	}
	// Shift the read command bits out.
	for (i =17; i >= 0; i--) {
		int dataval =(read_cmd & (1 << i)) ? MDIO_DATA_WRITE1 :0;

		outl(dataval, mdio_addr);
		mdio_delay();
		outl(dataval |MDIO_SHIFT_CLK, mdio_addr);
		mdio_delay();
		outl(dataval, mdio_addr);
		mdio_delay();
	}
	outl(MDIO_ENB_IN |MDIO_SHIFT_CLK, mdio_addr);
	mdio_delay();
	outl(MDIO_ENB_IN, mdio_addr);

	for (i =16; i > 0; i--) {
		outl(MDIO_ENB_IN |MDIO_SHIFT_CLK, mdio_addr);
		mdio_delay();
		retval =(retval <<1) |((inl(mdio_addr) &MDIO_DATA_READ) ? 1 :0);
		outl(MDIO_ENB_IN, mdio_addr);
		mdio_delay();
	}
	// Clear out extra bits.
	for (i =16; i > 0; i--) {
		outl(MDIO_ENB_IN |MDIO_SHIFT_CLK, mdio_addr);
		mdio_delay();
		outl(MDIO_ENB_IN, mdio_addr);
		mdio_delay();
	}
	return retval;
}

/*
// Known cards that have old-style EEPROMs.
static struct fixups {
  char *name;
  unsigned char addr0, addr1, addr2;
  u16 newtable[32];			// Max length below.
} srom_fixups[] ={
  {"Asante", 0, 0, 0x94, {0x1e00, 0, 0x800, 0x100, 0x18c, 0, 0,
			0xe078, 0x0001, 0x0050, 0x0018 }},
  {"SMC9332DST", 0, 0, 0xC0, {0x1e00, 0x0000, 0x0800, 0x021f,
			   0x0000, 0x009E,	// 10baseT
			   0x0903, 0x006D,}},	// 100baseTx
  {"Cogent EM100", 0, 0, 0x92, {0x1e00, 0x0000, 0x0800, 0x013f,
				0x0103, 0x006D,}},	// 100baseTx
  {"Maxtech NX-110", 0, 0, 0xE8, {0x1e00, 0x0000, 0x0800, 0x0313,
				0x1001, 0x009E, // 10base2, CSR12 0x10
				0x0000, 0x009E, // 10baseT
				0x0303, 0x006D,}},	// 100baseTx, CSR12 0x03
  {"Accton EN1207", 0, 0, 0xE8, {0x1e00, 0x0000, 0x0800, 0x031F,
				0x1B01, 0x0000, // 10base2,   CSR12 0x1B
				0x1B03, 0x006D, // 100baseTx, CSR12 0x1B
				0x0B00, 0x009E,}},	// 10baseT, CSR12 0x0B
  {0, 0, 0, 0, {}}};	*/

static const char * block_name[] ={"AX88140 non-MII", "AX88140 MII PHY",
 "AX88140 non-MII PHY", "AX88140 MII PHY", };

static void parse_srom(struct device *dev)
{
	// The last media info list parsed, for multiport boards.
	static struct mediatable *last_mediatable =NULL;
	static controller_index =0;
	struct ax88140_private *tp =(struct ax88140_private *)dev->priv;
	int ioaddr =dev->base_addr;
	unsigned char *ee_data =tp->sromdata;
	int i;

	tp->mtable =0;
	for (i =0; i < EEPROM_SIZE/2; i++)
	  ((u16 *)ee_data)[i] =read_srom(ioaddr, i);

	/*
	// Detect an old-style (SA only) EEPROM layout:
	   memcmp(eedata, eedata+16, 8).
	*/


	#if AX88140_DEBUG > 1
	printk(KERN_DEBUG "\nread_srom:");
	for (i =0; i < 64; i++) {
		printk("%s%4.4x", (i & 7) == 0 ? "\n" :" ",
			read_srom(ioaddr, i));
	}
	printk("\n");
	#endif

	controller_index =0;

//subsequent_board:

	{
		unsigned char *p =(void *)ee_data +ee_data[27];
		unsigned char csr12dir =0;
		int count;
		struct mediatable *mtable;
		short media =*((u16 *)p)++;

	//	if (tp->chip_id == DC21140)
			csr12dir =*p++;
		count =*p++;
		mtable =(struct mediatable *)
			kmalloc(sizeof(struct mediatable) +count*sizeof(struct medialeaf),
				GFP_KERNEL);
		if (mtable ==NULL)
			return; 	// Horrible, impossible failure.
		last_mediatable =tp->mtable =mtable;
		mtable->defaultmedia =media;
		mtable->leafcount =count;
		mtable->csr12dir =csr12dir;
		mtable->has_mii =0;

		printk(KERN_INFO"%s: EEPROM default media type %s.\n", dev->name,
			   media & 0x0800 ? "Autosense" :medianame[media & 15]);
		for (i =0; i < count; i++) {
			struct medialeaf *leaf =&mtable->mleaf[i];

			if ((p[0] & 0x80) == 0) { // 21140 Compact block.
				leaf->type =0;
				leaf->media =p[0] & 0x3f;
				leaf->leafdata =p;
				p += 4;
			} else {
				leaf->type =p[1];
				if (p[1] & 1) {
					mtable->has_mii =1;
					leaf->media =11;
				} else
					leaf->media =p[2] & 0x0f;
				leaf->leafdata =p +2;
				p += (p[0] & 0x3f) +1;
			}
			#if AX88140_DEBUG > 1
			if (leaf->media == 11) {
				unsigned char *bp =leaf->leafdata;
				printk(KERN_INFO "%s:  MII interface PHY %d, setup/reset "
					   "sequences %d/%d long, capabilities %2.2x %2.2x.\n",
					   dev->name, bp[0], bp[1], bp[1 +bp[1]*2],
					   bp[5 +bp[2 +bp[1]*2]*2], bp[4 +bp[2 +bp[1]*2]*2]);
				#if AX88140_DEBUG > 2
				{
					int mii_reg;
					printk(KERN_DEBUG "%s:  MII xcvr control registers:",
						   dev->name);
					for (mii_reg =0; mii_reg < 32; mii_reg++)
						printk(" %4.4x", mdio_read(ioaddr,bp[0], mii_reg));
					printk(".\n");
				}
				#endif
			}
			#endif

			printk(KERN_INFO"%s: Index #%d - Media %s (#%d) described "
				"by a %s (%d) block.\n", dev->name,
				i, medianame[leaf->media], leaf->media,
				block_name[leaf->type], leaf->type);
		}
	}
}
// Reading a serial EEPROM is a "bit" grungy, but we work our way through:->.


#include	"crc.h"
static inline unsigned ether_crc_le(int length, unsigned char *data)
{
	unsigned int crc =0xffffffff;	// Initial value.
	while (length--)
		crc =CRCTable[(crc ^ *data++) & 0xff] ^ (crc >> 8);
	return ~crc;
}

// A list of all installed devices, for removing the driver module.
#ifdef MODULE
static struct device *root_ax88140_dev =NULL;
#endif

// Set up the transceiver control registers for the selected media type.
static void select_media(struct device *dev, int startup)
{
	int ioaddr =dev->base_addr;
	struct ax88140_private *tp =(struct ax88140_private *)dev->priv;
	struct mediatable *mtable =tp->mtable;
	u32 new_csr6;
	int check_mii =0;

	if (mtable) {
		struct medialeaf *mleaf =&mtable->mleaf[tp->cur_index];
		unsigned char *p =mleaf->leafdata;
		int i;

		switch (mleaf->type) {
		case 0: 		// 21140 non-MII xcvr.
			#if AX88140_DEBUG > 1
			printk(KERN_DEBUG "%s: Using a AX88140 non-MII transceiver with control"
				" setting %2.2x.\n", dev->name, p[1]);
			#endif
			dev->if_port =p[0];
			if (startup)
				outl(mtable->csr12dir |0x100, ioaddr +CSR12);
			outl(p[1], ioaddr +CSR12);
			new_csr6 =0x02000000 |((p[2] &0x71) <<18);
			break;
		case 1:
			if (startup) {
				outl(mtable->csr12dir |0x100, ioaddr +CSR12);
				dev->if_port =11;
				#if AX88140_DEBUG >2
				printk(KERN_DEBUG "%s: Doing a reset sequence of length %d.\n",
					dev->name, p[2 +p[1]]);
				#endif
				for (i =0; i < p[2 +p[1]]; i++)
					outl(p[3 +p[1] +i], ioaddr +CSR12);
				#if AX88140_DEBUG >2
				printk(KERN_DEBUG "%s  Doing a transceiver setup sequence of length %d.\n",
					dev->name, p[1]);
				#endif
				for (i =0; i <p[1]; i++)
					outl(p[2 +i], ioaddr +CSR12);
			}
			check_mii =1;
			new_csr6 =0x020C0000;
			break;
		case 2: case 4: {
			u16 *setup =(u16*)&p[1];

			dev->if_port =p[0] & 15;
			#if AX88140_DEBUG > 1
			printk(KERN_DEBUG "%s: AX88140 non-MII %s transceiver control %4.4x/%4.4x.\n",
				dev->name, medianame[dev->if_port], setup[0], setup[1]);
			#endif
			/*	//bug!
			if (p[0] & 0x40) {	// SIA (CSR13-15) setup values are provided.
				outl(0, ioaddr +CSR13);
				outl(setup[1], ioaddr +CSR14);
				outl(setup[2], ioaddr +CSR15);
				outl(setup[0], ioaddr +CSR13);
				setup += 3;
			} else {
				outl(0, ioaddr +CSR13);
				outl(t21142_csr14[dev->if_port], ioaddr +CSR14);
				outl(t21142_csr15[dev->if_port], ioaddr +CSR15);
				outl(t21142_csr13[dev->if_port], ioaddr +CSR13);
			}
			*/	//bug!
			outl(setup[0]<<16, ioaddr +CSR15);	// Direction
			outl(setup[1]<<16, ioaddr +CSR15);	// Data
			new_csr6 =0x02000000;
			break;
		}
		case 3: {
			int init_length =p[1];
			u16 * init_sequence =(u16*)(p +2);
			int reset_length =p[2 +init_length*2];
			u16 * reset_sequence =(u16*)&p[3 +init_length*2];

			dev->if_port =11;
			if (startup) {
				#if AX88140_DEBUG > 2
				printk(KERN_DEBUG "%s:  Doing a AX88140 reset sequence of length %d.\n",
					dev->name, reset_length);
				#endif
				for (i =0; i < reset_length; i++)
					outl(reset_sequence[i] << 16, ioaddr +CSR15);
			}
			#if AX88140_DEBUG > 2
			printk(KERN_DEBUG "%s: Doing a AX88140 xcvr setup sequence of length %d.\n",
				 dev->name, init_length);
			#endif
			for (i =0; i < init_length; i++)
				outl(init_sequence[i] << 16, ioaddr +CSR15);
			check_mii =1;
			new_csr6 =0x020C0000;
			break;
		}
		default:
		  new_csr6 =0x020C0000;
		}
		#if AX88140_DEBUG > 1
		printk(KERN_DEBUG "%s: Using media type %s, CSR12 is %2.2x.\n",
			dev->name, medianame[dev->if_port], inl(ioaddr +CSR12) & 0xff);
		#endif
	} else {
		// Set media type to MII @ 100mbps: 0x020C0000
		new_csr6 =0x020C0000;
		dev->if_port =11;
		#if AX88140_DEBUG > 1
		printk(KERN_DEBUG "%s: Unknown media control, assuming MII, CSR12 %2.2x.\n",
			dev->name, inl(ioaddr +CSR12) & 0xff);
		#endif
	}
	tp->csr6 =new_csr6 |(tp->csr6 & 0xfdff) |(tp->full_duplex ? 0x0200 :0);
	return;
}

static void ax88140_timer(unsigned long data)
{
	struct device *dev =(struct device *)data;
	struct ax88140_private *tp =(struct ax88140_private *)dev->priv;
	int ioaddr =dev->base_addr;
	u32 csr12 =inl(ioaddr +CSR12);
	int next_tick =0;

	#if AX88140_DEBUG > 3
	printk(KERN_DEBUG "%s: Media selection tick, status %8.8x mode %8.8x "
		"SIA %8.8x %8.8x %8.8x %8.8x.\n", dev->name, inl(ioaddr +CSR5), inl(ioaddr +CSR6),
		csr12, inl(ioaddr +CSR13), inl(ioaddr +CSR14), inl(ioaddr +CSR15));
	#endif
	{
//	switch(tp->chip_id) {
//	case DC21140:  case DC21142: {
		struct medialeaf *mleaf;
		unsigned char *p;
		if (tp->mtable ==NULL) {	//No SROM info, use generic code
			// Assume this is like a SMC card, and check its link beat bit.
			if ((dev->if_port == 0 && (csr12 & 0x0080)) ||
				(dev->if_port == 1 && (csr12 & 0x0040) == 0)) {
				dev->if_port ^= 1;
				// Stop the transmit process.
				tp->csr6 =(dev->if_port ? 0x03860000 :0x02420000);
				outl(tp->csr6 |0x0002, ioaddr +CSR6);
				printk(KERN_INFO "%s: link beat timed out, CSR12 is 0x%2.2x, switching to"
					   " %s media.\n", dev->name,
					   csr12 & 0xff,
					   dev->if_port ? "100baseTx" :"10baseT");
				outl(tp->csr6 |0xA002, ioaddr +CSR6);
				dev->trans_start =jiffies;
				next_tick =(24*HZ)/10;
			} else {
				next_tick =10*HZ;
				#if AX88140_DEBUG > 2
				printk(KERN_DEBUG "%s: network media monitor 0x%2.2x, link"
					" beat detected as %s.\n", dev->name, csr12 & 0xff,
					dev->if_port ? "100BaseTx" :"10BaseT");
				#endif
			}
			goto ax88140_timer_break;
		}
	mleaf =&tp->mtable->mleaf[tp->cur_index];
	p =mleaf->leafdata;
	switch (mleaf->type) {
	case 0: case 4: {
		// Type 0 non-MII or #4 SYM transceiver. Check the link beat bit
		s8 bitnum =p[mleaf->type == 4 ? 5 :2];
		#if AX88140_DEBUG > 2
		printk(KERN_DEBUG "%s: Transceiver monitor tick: CSR12=%#2.2x bit %d is"
			" %d, expecting %d.\n", dev->name, csr12, (bitnum >> 1) & 7,
			 (csr12 & (1 << ((bitnum >> 1) & 7))) != 0, (bitnum >= 0));
		#endif
		// Check that the specified bit has the proper value.
		if ((bitnum < 0) != ((csr12 & (1 << ((bitnum >> 1) & 7))) != 0)) {
			#if AX88140_DEBUG > 1
			printk(KERN_DEBUG "%s: Link beat detected for %s.\n", dev->name,
				medianame[mleaf->media]);
			#endif
			break;
		}
		if (tp->medialock)
			break;

		do {
			if (--tp->cur_index < 0) {
				// We start again, but should instead look for default.
				tp->cur_index =tp->mtable->leafcount -1;
			}
			dev->if_port =tp->mtable->mleaf[tp->cur_index].media;
		} while (media_fd[dev->if_port]);	// Skip FD entries.
		#if AX88140_DEBUG > 1
		printk(KERN_DEBUG "%s: No link beat on media %s, trying transceiver type %s.\n",
				 dev->name, medianame[mleaf->media & 15],
				 medianame[tp->mtable->mleaf[tp->cur_index].media]);
		#endif
		select_media(dev, 0);
		// Restart the transmit process.
		outl(tp->csr6 |0x0002, ioaddr +CSR6);
		outl(tp->csr6 |0x2002, ioaddr +CSR6);
		next_tick =(24*HZ)/10;
		break;
	  }
	  case 1:
		{
			int mii_reg5 =mdio_read(ioaddr, tp->phys[0], 5);
			printk(KERN_INFO "%s: MII monitoring tick: CSR12 %2.2x, "
				"Link partner report %4.4x.\n", dev->name, csr12, mii_reg5);
			if (mii_reg5 != 0xffff && mdio_read(ioaddr, tp->phys[0], 1) & 0x0020) {
				int full_duplex =mii_reg5 & 0x0100 ? 1 :0;
				if (full_duplex != tp->full_duplex) {
					tp->full_duplex =full_duplex;
					tp->csr6 ^= 0x0200;
					outl(tp->csr6 |0x0002, ioaddr +CSR6);
					outl(tp->csr6 |0x2002, ioaddr +CSR6);
				}
				#if AX88140_DEBUG > 0	// Gurppp, should be >1
				printk(KERN_INFO "%s: Setting %s-duplex based on MII"
					" Xcvr #%d parter capability of %4.4x.\n",
					dev->name, full_duplex ? "full" :"half",
					tp->phys[0], mii_reg5);
				#endif
			}
		}

		// Hack for D-Link: Full duplex indication is on bit 3.
		if (dev->dev_addr[0] == 0  &&  dev->dev_addr[1] == 0x80
			&& dev->dev_addr[2] == 0xC8) {
			if (csr12 & 0x08) {
				tp->full_duplex =0;
				tp->csr6 &= ~0x0200;
				outl(tp->csr6 |0x0002, ioaddr +CSR6);
				outl(tp->csr6 |0x2002, ioaddr +CSR6);
			} else {
				tp->full_duplex =1;
				tp->csr6 |= 0x0200;
				outl(tp->csr6 |0x0002, ioaddr +CSR6);
				outl(tp->csr6 |0x2002, ioaddr +CSR6);
			}
		}
		break;
	  case 2:		// 21142 non-MII
	  case 3:		// 21142 MII
		next_tick =(24*HZ)/10;
		break;
	  default:
		break;
	  }
//	}
	}
ax88140_timer_break:
	if (next_tick) {
		tp->timer.expires =RUN_AT(next_tick);
		add_timer(&tp->timer);
	}
}


// Initialize the Rx and Tx rings, along with various 'dev' bits.
static void
ax88140_init_ring(struct device *dev)
{
	struct ax88140_private *tp =(struct ax88140_private *)dev->priv;
	int i;

	tp->tx_full =0;
	tp->cur_rx =tp->cur_tx =0;
	tp->dirty_rx =tp->dirty_tx =0;

	for (i =0; i < RX_RING_SIZE; i++) {
		tp->rx_ring[i].status =0x80000000;	// Owned by chip
		tp->rx_ring[i].length =PKT_BUF_SZ;
		{
			/* Note the receive buffer must be longword aligned.
			   dev_alloc_skb() provides 16 byte alignment.	But
			   do *not* use skb_reserve() to align the IP header! */
			struct sk_buff *skb;
			skb =DEV_ALLOC_SKB(PKT_BUF_SZ);
			tp->rx_skbuff[i] =skb;
			if (skb == NULL)
				break;	// Bad news!
			skb->dev =dev;	// Mark as being used by this device.
#if LINUX_VERSION_CODE > 0x10300
			tp->rx_ring[i].buffer1 =virt_to_bus(skb->tail);
#else
			tp->rx_ring[i].buffer1 =virt_to_bus(skb->data);
#endif
		}
		tp->rx_ring[i].buffer2 =virt_to_bus(&tp->rx_ring[i+1]);
	}
	// Mark the last entry as wrapping the ring.
	tp->rx_ring[i-1].length =PKT_BUF_SZ |0x02000000;
	tp->rx_ring[i-1].buffer2 =virt_to_bus(&tp->rx_ring[0]);

	/* The Tx buffer descriptor is filled in as needed, but we
	   do need to clear the ownership bit. */
	for (i =0; i < TX_RING_SIZE; i++) {
		tp->tx_skbuff[i] =0;
		tp->tx_ring[i].status =0x00000000;
		tp->tx_ring[i].buffer2 =virt_to_bus(&tp->tx_ring[i+1]);
	}
	tp->tx_ring[i-1].buffer2 =virt_to_bus(&tp->tx_ring[0]);
}


static int
ax88140_rx(struct device *dev)
{
	struct ax88140_private *lp =(struct ax88140_private *)dev->priv;
	int entry =lp->cur_rx % RX_RING_SIZE;

	#if AX88140_DEBUG > 4
	printk(KERN_DEBUG " In ax88140_rx(), entry %d %8.8x.\n", entry,
		lp->rx_ring[entry].status);
	#endif
	// If we own the next entry, it's a new packet. Send it up.
	while (lp->rx_ring[entry].status >= 0) {
		int status =lp->rx_ring[entry].status;

		if ((status & 0x0300) != 0x0300) {
			if ((status & 0xffff) != 0x7fff) { // Ingore earlier buffers.
			  printk(KERN_WARNING "%s: Oversized Ethernet frame spanned "
					 "multiple buffers, status %8.8x!\n", dev->name, status);
			  lp->stats.rx_length_errors++;
			}
		} else if (status & 0x8000) {
			// There was a fatal error.
			lp->stats.rx_errors++; // end of a packet.
			if (status & 0x0890) lp->stats.rx_length_errors++;
			if (status & 0x0004) lp->stats.rx_frame_errors++;
			if (status & 0x0002) lp->stats.rx_crc_errors++;
			if (status & 0x0001) lp->stats.rx_fifo_errors++;
		} else {
			// Malloc up new buffer, compatible with net-2e.
			// Omit the four octet CRC from the length.
			short pkt_len =(lp->rx_ring[entry].status >> 16) -4;
			struct sk_buff *skb;
			int rx_in_place =0;

			/* Check if the packet is long enough to just accept without
			   copying to a properly sized skbuff. */
			if (pkt_len > rx_copybreak) {
				struct sk_buff *newskb;
				char *temp;

				// Get a fresh skbuff to replace the filled one.
				newskb =DEV_ALLOC_SKB(PKT_BUF_SZ);
				if (newskb == NULL) {
					skb =0;	// No memory, drop the packet.
					goto memory_squeeze;
				}
				// Pass up the skb already on the Rx ring.
				skb =lp->rx_skbuff[entry];
				temp =skb_put(skb, pkt_len);
				if (bus_to_virt(lp->rx_ring[entry].buffer1) != temp)
					printk(KERN_ERR "%s: Internal consistency error -- the "
						   "skbuff addresses do not match"
						   " in ax88140_rx: %p vs. %p / %p.\n", dev->name,
						   bus_to_virt(lp->rx_ring[entry].buffer1),
						   skb->head, temp);
				rx_in_place =1;
				lp->rx_skbuff[entry] =newskb;
				newskb->dev =dev;
				// Longword alignment required: do not skb_reserve(2)!
				lp->rx_ring[entry].buffer1 =virt_to_bus(newskb->tail);
			} else
				skb =DEV_ALLOC_SKB(pkt_len +2);
			memory_squeeze:
			if (skb == NULL) {
				int i;
				printk(KERN_WARNING "%s: Memory squeeze, deferring packet.\n",
					   dev->name);
				/* Check that at least two ring entries are free.
				   If not, free one and mark stats->rx_dropped++. */
				for (i =0; i < RX_RING_SIZE; i++)
					if (lp->rx_ring[(entry+i) % RX_RING_SIZE].status < 0)
						break;

				if (i > RX_RING_SIZE -2) {
					lp->stats.rx_dropped++;
					lp->rx_ring[entry].status =0x80000000;
					lp->cur_rx++;
				}
				break;
			}
			skb->dev =dev;
			if (! rx_in_place) {
				skb_reserve(skb, 2);	// 16 byte align the data fields
				memcpy(skb_put(skb, pkt_len),
					   bus_to_virt(lp->rx_ring[entry].buffer1), pkt_len);
			}
#if LINUX_VERSION_CODE > 0x10300
			skb->protocol =eth_type_trans(skb, dev);
#else
			skb->len =pkt_len;
#endif
			netif_rx(skb);
			lp->stats.rx_packets++;
		}
		lp->rx_ring[entry].status =0x80000000;
		entry =(++lp->cur_rx) % RX_RING_SIZE;
	}
	return 0;
}


/* The interrupt handler does all of the Rx thread work and cleans up
   after the Tx thread. */
static void ax88140_interrupt IRQ(int irq, void *dev_instance, struct pt_regs *regs)
{
#ifdef SA_SHIRQ 	// Use the now-standard shared IRQ implementation.
	struct device *dev =(struct device *)dev_instance;
#else
	struct device *dev =(struct device *)(irq2dev_map[irq]);
#endif

	struct ax88140_private *lp;
	int csr5, ioaddr, boguscnt =12;

	if (dev == NULL) {
		printk ("ax88140_interrupt(): irq %d for unknown device.\n", irq);
		return;
	}

	ioaddr =dev->base_addr;
	lp =(struct ax88140_private *)dev->priv;
	if (dev->interrupt)
		printk(KERN_ERR "%s: Re-entering the interrupt handler.\n", dev->name);

	dev->interrupt =1;

	do {
		csr5 =inl(ioaddr +CSR5);
		// Acknowledge all of the current interrupt sources ASAP.
		outl(csr5 & 0x0001ffff, ioaddr +CSR5);

		#if AX88140_DEBUG > 4
		printk(KERN_DEBUG "%s: interrupt  csr5=%#8.8x new csr5=%#8.8x.\n",
			dev->name, csr5, inl(dev->base_addr +CSR5));
		#endif

		if ((csr5 & 0x00018000) == 0)
			break;

		if (csr5 & 0x0040)		// Rx interrupt
			ax88140_rx(dev);

		if (csr5 & 0x0007) {		// Tx-done interrupt
			int dirty_tx;

			for (dirty_tx =lp->dirty_tx; dirty_tx < lp->cur_tx; dirty_tx++) {
				int entry =dirty_tx % TX_RING_SIZE;
				int status =lp->tx_ring[entry].status;

				if (status < 0)
					break;	// It still hasn't been Txed
				// Check for Rx filter setup frames.
				if (lp->tx_skbuff[entry] == NULL)
				  continue;

				if (status & 0x8000) {
					// There was an major error, log it.
#ifndef final_version
					#if AX88140_DEBUG > 1
					printk(KERN_DEBUG "%s: Transmit error, Tx status %8.8x.\n",
						dev->name, status);
					#endif
#endif
					lp->stats.tx_errors++;
					if (status & 0x4104) lp->stats.tx_aborted_errors++;
					if (status & 0x0C00) lp->stats.tx_carrier_errors++;
					if (status & 0x0200) lp->stats.tx_window_errors++;
					if (status & 0x0002) lp->stats.tx_fifo_errors++;
					if ((status & 0x0080) && lp->full_duplex == 0)
						lp->stats.tx_heartbeat_errors++;
#ifdef ETHER_STATS
					if (status & 0x0100) lp->stats.collisions16++;
#endif
				} else {
#ifdef ETHER_STATS
					if (status & 0x0001) lp->stats.tx_deferred++;
#endif
					lp->stats.collisions += (status >> 3) & 15;
					lp->stats.tx_packets++;
				}

				// Free the original skb.
				dev_kfree_skb(lp->tx_skbuff[entry], FREE_WRITE);
				lp->tx_skbuff[entry] =0;
			}

#ifndef final_version
			if (lp->cur_tx -dirty_tx > TX_RING_SIZE) {
				printk(KERN_ERR "%s: Out-of-sync dirty pointer, %d vs. %d, full=%d.\n",
					   dev->name, dirty_tx, lp->cur_tx, lp->tx_full);
				dirty_tx += TX_RING_SIZE;
			}
#endif

			if (lp->tx_full && dev->tbusy
				&& dirty_tx > lp->cur_tx -TX_RING_SIZE +2) {
				// The ring is no longer full, clear tbusy.
				lp->tx_full =0;
				dev->tbusy =0;
				mark_bh(NET_BH);
			}

			lp->dirty_tx =dirty_tx;
		}

		// Log errors.
		if (csr5 & 0x8000) {	// Abnormal error summary bit.
			if (csr5 & 0x0008) lp->stats.tx_errors++; // Tx babble.
			if (csr5 & 0x0020) {	// Tx FIFO underflow.
			  lp->csr6 |= 0x00200000; // Reconfigure to store-n-forward.
			  // Restart the transmit process.
			  outl(lp->csr6 |0x0002, ioaddr +CSR6);
			  outl(lp->csr6 |0x2002, ioaddr +CSR6);
			}
			if (csr5 & 0x0100) {	// Missed a Rx frame.
				lp->stats.rx_errors++;
				lp->stats.rx_missed_errors += inl(ioaddr +CSR8) & 0xffff;
			}
			if (csr5 & 0x0800) {
				printk(KERN_ERR "%s: Something Wicked happened! %8.8x.\n",
					   dev->name, csr5);
				// Hmmmmm, it's not clear what to do here.
			}
			// Clear all error sources, included undocumented ones!
			outl(0x000f7ba, ioaddr +CSR5);
		}
		if (--boguscnt < 0) {
			printk(KERN_WARNING "%s: Too much work at interrupt, csr5=0x%8.8x.\n",
				   dev->name, csr5);
			// Clear all interrupt sources.
			outl(0x0001ffff, ioaddr +CSR5);
			break;
		}
	} while (1);

	#if AX88140_DEBUG > 3
	printk(KERN_DEBUG "%s: exiting interrupt, csr5=%#4.4x.\n",
		dev->name, inl(ioaddr +CSR5));
	#endif

	// Code that should never be run!  Perhaps remove after testing..
	{
		static int stopit =10;
		if (dev->start == 0  &&  --stopit < 0) {
			printk(KERN_ERR "%s: Emergency stop, looping startup interrupt.\n"
				   KERN_ERR "%s: Disabling interrupt handler %d to avoid "
				   "locking up the machine.\n",
				   dev->name, dev->name, dev->irq);
#ifdef SA_SHIRQ
			free_irq(irq, dev);
#else
			free_irq(irq);
#endif
		}
	}

	dev->interrupt =0;
	return;
}


static int ax88140_open(struct device *dev)
{
	struct ax88140_private *tp =(struct ax88140_private *)dev->priv;
	int ioaddr =dev->base_addr;
	int i =0;

	// On some chip revs we must set the MII/SYM port before the reset!?
	if (tp->mtable && tp->mtable->has_mii)
		outl(0x00040000, ioaddr +CSR6);

	// Reset the chip, holding bit 0 set at least 50 PCI cycles.
	outl(0x00000001, ioaddr +CSR0);
#ifdef _LINUX_DELAY_H
	udelay(2);
#else
	SLOW_DOWN_IO;
#endif
	/*
	  Deassert reset.
	  486: Set 8 longword cache alignment, 8 longword burst.
	  586: Set 16 longword cache alignment, no burst limit.
	  Cache alignment bits 15:14	     Burst length 13:8
	0000	No alignment	0000 unlimited		0800 8 longwords
	4000	8 longwords	0100 1 longword 	1000 16 longwords
	8000	16 longwords	0200 2 longwords	2000 32 longwords
	C000	32  longwords	0400 4 longwords
	   Wait the specified 50 PCI cycles after a reset by initializing
	   Tx and Rx queues and the address filter list.
	*/

#if defined(__alpha__)
	// ToDo: Alpha setting could be better.
	outl(0x00200000 |0xE000, ioaddr +CSR0);
#else
#if defined(MODULE)
	// When a module we don't have 'x86' to check.
	outl(0x00202000 , ioaddr +CSR0);
#else
	outl(0x00202000 , ioaddr +CSR0);
	if (x86 <= 4)
		printk(KERN_INFO "%s: This is a 386/486 PCI system, setting"
			 " cache alignment to 0x00202000.\n", dev->name);
#endif
#endif

#ifdef SA_SHIRQ
	if (request_irq(dev->irq, &ax88140_interrupt, SA_SHIRQ,
			ax88140_tbl[tp->chip_id].chip_name, dev))
#else
	if (irq2dev_map[dev->irq] != NULL
		|| (irq2dev_map[dev->irq] =dev) == NULL
		|| dev->irq == 0
		|| request_irq(dev->irq, &ax88140_interrupt, 0,
			   ax88140_tbl[tp->chip_id].chip_name))
#endif
		return -EAGAIN;

	#if AX88140_DEBUG > 1
	printk(KERN_DEBUG "%s: ax88140_open() irq %d.\n", dev->name, dev->irq);
	#endif

	MOD_INC_USE_COUNT;

	ax88140_init_ring(dev);
	outl(virt_to_bus(tp->rx_ring), ioaddr +CSR3);
	outl(virt_to_bus(tp->tx_ring), ioaddr +CSR4);


	// ASIX style!
	// set_rx_mode() without starting the transmitter.
	{
		outl(tp->csr6 |=0x100, ioaddr +CSR6);
		outl(0, ioaddr +CSR13);
		outl(*((u32 *)dev->dev_addr), ioaddr +CSR14);
		outl(1, ioaddr +CSR13);
		outl(*((u32 *) &dev->dev_addr[4]) & 0xffff, ioaddr +CSR14);
	}

	if (dev->if_port == 0)
		dev->if_port =tp->default_port;

	// Invalid: Select initial TP, autosense, autonegotiate.
	if (dev->if_port > 4)
		dev->if_port =4;

	// Allow selecting a default media.
	if (tp->mtable ==0)
		goto media_picked;
	if (dev->if_port)
		for (i =0; i < tp->mtable->leafcount; i++)
		  if (tp->mtable->mleaf[i].media ==
			  (dev->if_port == 12 ? 0 :dev->if_port)) {
			printk(KERN_INFO "%s: Using user-specified media %s.\n",
				   dev->name, medianame[dev->if_port]);
			goto media_picked;
		  }
	if ((tp->mtable->defaultmedia & 0x0800) == 0)
		for (i =0; i < tp->mtable->leafcount; i++)
		  if (tp->mtable->mleaf[i].media == (tp->mtable->defaultmedia & 15)) {
			printk(KERN_INFO "%s: Using EEPROM-set media %s.\n",
				   dev->name, medianame[tp->mtable->mleaf[i].media]);
			goto media_picked;
		  }
	for (i =tp->mtable->leafcount -1;
		 (media_fd[tp->mtable->mleaf[i].media] & 2) && i > 0; i--)
	  ;
media_picked:

	tp->cur_index =i;
	tp->csr6 =0;
	select_media(dev, 1);

	// Start the chip's Tx to process setup frame.
	outl(tp->csr6, ioaddr +CSR6);
	outl(tp->csr6 |0x2000, ioaddr +CSR6);

	dev->tbusy =0;
	dev->interrupt =0;
	dev->start =1;

	// Enable interrupts by setting the interrupt mask.
	outl(0x0001fbff, ioaddr +CSR7);
	outl(tp->csr6 |0x2002, ioaddr +CSR6);
	outl(0, ioaddr +CSR2); 	// Rx poll demand

	#if AX88140_DEBUG > 2
	printk(KERN_DEBUG "%s: Done ax88140_open(), CSR0 %8.8x, CSR5 %8.8x CSR13 %8.8x.\n",
		dev->name, inl(ioaddr +CSR0), inl(ioaddr +CSR5), inl(ioaddr +CSR13));
	#endif

        /* Set the timer to switch to check for link beat and perhaps switch
	   to an alternate media type. */
	init_timer(&tp->timer);
	tp->timer.expires =RUN_AT((24*HZ)/10);	// 2.4 sec.
	tp->timer.data =(unsigned long)dev;
	tp->timer.function =&ax88140_timer;	// timer handler
	add_timer(&tp->timer);

	return 0;
}

static int ax88140_close(struct device *dev)
{
	int ioaddr =dev->base_addr;
	struct ax88140_private *tp =(struct ax88140_private *)dev->priv;
	int i;

	dev->start =0;
	dev->tbusy =1;

	#if AX88140_DEBUG > 1
	printk(KERN_DEBUG "%s: AX88140 shutting down, CSR5: %2.2x.\n",
		dev->name, inl(ioaddr +CSR5));
	#endif

	// Disable interrupts by clearing the interrupt mask.
	outl(0x00000000, ioaddr +CSR7);
	// Stop the chip's Tx and Rx processes.
	outl(inl(ioaddr +CSR6) & ~0x2002, ioaddr +CSR6);

	tp->stats.rx_missed_errors += inl(ioaddr +CSR8) & 0xffff;

	del_timer(&tp->timer);

#ifdef SA_SHIRQ
	free_irq(dev->irq, dev);
#else
	free_irq(dev->irq);
	irq2dev_map[dev->irq] =0;
#endif

	// Free all the skbuffs in the Rx queue.
	for (i =0; i < RX_RING_SIZE; i++) {
		struct sk_buff *skb =tp->rx_skbuff[i];
		tp->rx_skbuff[i] =0;
		tp->rx_ring[i].status =0;	// Not owned by chip.
		tp->rx_ring[i].length =0;
		tp->rx_ring[i].buffer1 =0xBADF00D0;	// An invalid address.
		if (skb) {
#if LINUX_VERSION_CODE < 0x20100
			skb->free =1;
#endif
			dev_kfree_skb(skb, FREE_WRITE);
		}
	}
	for (i =0; i < TX_RING_SIZE; i++) {
		if (tp->tx_skbuff[i])
			dev_kfree_skb(tp->tx_skbuff[i], FREE_WRITE);
		tp->tx_skbuff[i] =0;
	}


	MOD_DEC_USE_COUNT;

	return 0;
}

static struct enet_statistics *ax88140_get_stats(struct device *dev)
{
	struct ax88140_private *tp =(struct ax88140_private *)dev->priv;
	int ioaddr =dev->base_addr;

	if (dev->start)
		tp->stats.rx_missed_errors += inl(ioaddr +CSR8) & 0xffff;

	return &tp->stats;
}

// Set or clear the multicast filter for this adaptor.
static void
#ifdef NEW_MULTICAST
set_multicast_list(struct device *dev)
#else
set_multicast_list(struct device *dev, int num_addrs, void *addrs);
#endif
{
	int ioaddr =dev->base_addr;
	int csr6 =inl(ioaddr +CSR6) & ~0x00D5;
	struct ax88140_private *tp =(struct ax88140_private *)dev->priv;

	tp->csr6 &= ~0x00D5;
	if (dev->flags & IFF_PROMISC) {
		// Set promiscuous.
		outl(csr6 |0xC0, ioaddr +CSR6);
		// Unconditionally log net taps.
		printk(KERN_INFO "%s: Promiscuous mode enabled.\n", dev->name);
		tp->csr6 |= 0xC0;
	} else if ((dev->mc_count >128) ||(dev->flags &IFF_ALLMULTI)) {
		// Too many to filter perfectly -- accept all multicasts.
		outl(csr6 |0x80, ioaddr +CSR6);
		tp->csr6 |= 0x80;
	} else {
		//multicast!!!
		struct dev_mc_list *mclist;
		int i;

		tp->multicast_bits[0] = 
			tp->multicast_bits[1] = 0;
		//Must use a hash table.
		for (i =0, mclist =dev->mc_list;
			(mclist !=NULL) && (i < dev->mc_count);
			i++, mclist =mclist->next) {
			int j, k, l;
			j =~ether_crc_le(ETH_ALEN, mclist->dmi_addr) &0x3f;
			for (k=l=0; k <6; k++)
				if (j & (1 <<k))
					l |= 1 <<(5-k);
			tp->multicast_bits[l/32] |= 1 << (l % 32);
		}
		outl(2, ioaddr +CSR13);
		outl(tp->multicast_bits[0], ioaddr +CSR14);
		outl(3, ioaddr +CSR13);
		outl(tp->multicast_bits[1], ioaddr +CSR14);
		// Too clever: i > 15 for fall-though.
		outl(csr6, ioaddr +CSR6);
	}
}

static void ax88140_tx_timeout(struct device *dev)
{
  struct ax88140_private *tp =(struct ax88140_private *)dev->priv;
  int ioaddr =dev->base_addr;

  if (tp->mtable && tp->mtable->has_mii) {
	// Do nothing -- the media monitor should handle this.
	#if AX88140_DEBUG > 1
	printk(KERN_WARNING "%s: Transmit timeout using MII device.\n", dev->name);
	#endif
  } else {
  	// DC21140 or DC21142
	// Stop the transmit process.
	outl(tp->csr6 |0x0002, ioaddr +CSR6);
	dev->if_port ^= 1;
	printk(KERN_WARNING "%s: AX88140 transmit timed out, status %8.8x, "
		   "SIA %8.8x %8.8x %8.8x %8.8x, resetting...\n",
		   dev->name, inl(ioaddr +CSR5), inl(ioaddr +CSR12),
		   inl(ioaddr +CSR13), inl(ioaddr +CSR14), inl(ioaddr +CSR15));
	printk(KERN_WARNING "%s: transmit timed out, switching to %s media.\n",
		   dev->name, dev->if_port ? "100baseTx" :"10baseT");
	outl(tp->csr6 |0x2002, ioaddr +CSR6);
	tp->stats.tx_errors++;
	dev->trans_start =jiffies;
	return;
  }
#ifdef way_too_many_messages
  printk("  Rx ring %8.8x: ", (int)tp->rx_ring);
  for (i =0; i < RX_RING_SIZE; i++)
	printk(" %8.8x", (unsigned int)tp->rx_ring[i].status);
  printk("\n  Tx ring %8.8x: ", (int)tp->tx_ring);
  for (i =0; i < TX_RING_SIZE; i++)
	printk(" %8.8x", (unsigned int)tp->tx_ring[i].status);
  printk("\n");
#endif

  // Perhaps we should reinitialize the hardware here.
  dev->if_port =0;
  // Stop and restart the chip's Tx processes.
  outl(tp->csr6 |0x0002, ioaddr +CSR6);
  outl(tp->csr6 |0x2002, ioaddr +CSR6);
  // Trigger an immediate transmit demand.
  outl(0, ioaddr +CSR1);

  dev->trans_start =jiffies;
  tp->stats.tx_errors++;
  return;
}

static int
ax88140_start_tx(struct sk_buff *skb, struct device *dev)
{
	struct ax88140_private *tp =(struct ax88140_private *)dev->priv;
	int entry;
	u32 flag;

#ifndef final_version
	if (skb == NULL || skb->len <= 0) {
		printk(KERN_ERR"%s: Obsolete driver layer request null buff.\n",
			dev->name);
		dev_tint(dev);
		return 0;
	}
#endif

	/* Block a timer-based transmit from overlapping.  This could better be
	   done with atomic_swap(1, dev->tbusy), but set_bit() works as well. */
	if (test_and_set_bit(0, (void*)&dev->tbusy) != 0) {
		if (jiffies -dev->trans_start < TX_TIMEOUT)
			return 1;
		ax88140_tx_timeout(dev);
		return 1;
	}

	/* Caution: the write order is important here, set the base address
	   with the "ownership" bits last. */

	// Calculate the next Tx descriptor entry.
	entry =tp->cur_tx % TX_RING_SIZE;

	tp->tx_skbuff[entry] =skb;
	tp->tx_ring[entry].buffer1 =virt_to_bus(skb->data);

	if (tp->cur_tx -tp->dirty_tx < TX_RING_SIZE/2) {	// Typical path
	  flag =0x60000000;	// No interrupt
	  dev->tbusy =0;
	} else if (tp->cur_tx -tp->dirty_tx == TX_RING_SIZE/2) {
	  flag =0xe0000000;	// Tx-done intr.
	  dev->tbusy =0;
	} else if (tp->cur_tx -tp->dirty_tx < TX_RING_SIZE -2) {
	  flag =0x60000000;	// No Tx-done intr.
	  dev->tbusy =0;
	} else {
	  // Leave room for set_rx_mode() to fill entries.
	  flag =0xe0000000;	// Tx-done intr.
	  tp->tx_full =1;
	}
	if (entry == TX_RING_SIZE-1)
		flag |= 0xe2000000;

	tp->tx_ring[entry].length =skb->len |flag;
	tp->tx_ring[entry].status =0x80000000; // Pass ownership to the chip.
	tp->cur_tx++;
	// Trigger an immediate transmit demand.
	outl(0, dev->base_addr +CSR1);

	dev->trans_start =jiffies;

	return 0;
}

static struct device *ax88140_probe1(struct device *dev, int ioaddr, int irq,
				int chip_id, int board_idx)
{
	static int did_version =0;	// Already printed version info.
	struct ax88140_private *tp;
	// See note below on the multiport cards.
	static unsigned char last_phys_addr[6] ={0x00,'L','i','n','u','x'};
	static int last_irq =0;
	int i;
//	unsigned short sum;

	if (did_version++ == 0)
		printk(KERN_INFO "%s", version);

	dev =init_etherdev(dev, 0);

	printk(KERN_INFO "%s: ASIX %s at %#3x, ",
		   dev->name, ax88140_tbl[chip_id].chip_name, ioaddr);

	// Stop the chip's Tx and Rx processes.
	outl(inl(ioaddr +CSR6) & ~0x2002, ioaddr +CSR6);
	// Clear the missed-packet counter.
	(volatile)inl(ioaddr +CSR8);


	// The station address ROM is read byte serially.  The register must
	// be polled, waiting for the value to be read bit serially from the
	// EEPROM.

//	sum =0;
	{
		// Must be a new chip, with a serial EEPROM interface.
		// We read the whole EEPROM, and sort it out later. ASIX has a
		// specification _ASIX Electronic AX88140 Serial ROM Format_
		// but early vendor boards just put the address in the first six
		// EEPROM locations.
		unsigned char ee_data[EEPROM_SIZE];
		int sa_offset =0;

		for (i =0; i < EEPROM_SIZE/2; i++)
			((u16 *)ee_data)[i] =read_srom(ioaddr, i);

		if ((ether_crc_le(EEPROM_SIZE-2, ee_data) & 0xffff) != ((u16 *)ee_data)[EEPROM_SIZE/2-1])
			printk("SROM checksum error, ");
		// Detect the EEPROM format by the duplicated station addr.
		for (i =0; i < 8; i ++)
			if (ee_data[i] != ee_data[16+i]) {
				sa_offset =20;
				break;
			}
		for (i =0; i < 6; i ++) {
			dev->dev_addr[i] =ee_data[i +sa_offset];
//			sum += ee_data[i +sa_offset];
		}
	}
	printk("MAC address ");
	for (i =0; i < 6; i++)
		printk("%2.2x", last_phys_addr[i] =dev->dev_addr[i]);
	printk(", IRQ %d.\n", irq);
	last_irq =irq;

	// We do a request_region() only to register /proc/ioports info.
	request_region(ioaddr, AX88140_TOTAL_SIZE, ax88140_tbl[chip_id].chip_name);

	dev->base_addr =ioaddr;
	dev->irq =irq;

	// Make certain the data structures are quadword aligned.
	tp =(void *)(((long)kmalloc(sizeof(*tp),GFP_KERNEL |GFP_DMA) +7) &~7);
	memset(tp, 0, sizeof(*tp));
	dev->priv =tp;

#ifdef MODULE
	tp->next_module =root_ax88140_dev;
	root_ax88140_dev =dev;
#endif

	tp->chip_id =chip_id;

#ifdef AX88140_FULL_DUPLEX
	tp->full_duplex =1;
#endif
#ifdef AX88140_DEFAULT_MEDIA
	tp->default_port =AX88140_DEFAULT_MEDIA;
#endif
#ifdef AX88140_NO_MEDIA_SWITCH
	tp->medialock =1;
#endif

	// The lower four bits are the media type.
	if (board_idx >= 0) {
		tp->full_duplex =(options[board_idx]&16) ||full_duplex[board_idx]>0;
		tp->default_port =options[board_idx] & 15;
		if (tp->default_port)
			tp->medialock =1;
	}

	// This is logically part of probe1(), but too complex to write inline.
	parse_srom(dev);

	if (tp->mtable && tp->mtable->has_mii) {
		int phy, phy_idx;
		//Find the connected MII xcvrs.
		//Doing this in open() would allow detecting external xcvrs
		// later, but takes much time.
		for (phy =0, phy_idx =0; phy <32 &&phy_idx <sizeof(tp->phys);
			phy++) {
			int mii_status =mdio_read(ioaddr, phy, 0);
			if (mii_status != 0xffff  && mii_status != 0x0000) {
				tp->phys[phy_idx++] =phy;
				printk(KERN_INFO"%s: MII transceiver found at MDIO address %d.\n",
					dev->name, phy);
			}
		}
		if (phy_idx == 0) {
			printk(KERN_INFO "%s: WOW! No MII transceiver found!\n",
				dev->name);
			tp->phys[0] =1;
		}
	}

	// The driver specific entries in the device structure.
	dev->open =&ax88140_open;
	dev->hard_start_xmit =&ax88140_start_tx;
	dev->stop =&ax88140_close;
	dev->get_stats =&ax88140_get_stats;
#ifdef HAVE_MULTICAST
	dev->set_multicast_list =&set_multicast_list;
#endif

	// Reset the xcvr interface and turn on heartbeat.
	{
	// case DC21140:  case DC21142:
		if (tp->mtable)
			outl(tp->mtable->csr12dir |0x100, ioaddr +CSR12);
	//	break;
	}

	return dev;
}


/* This 21040 probe no longer uses a large fixed contiguous Rx buffer region,
   but now receives directly into full-sized skbuffs that are allocated
   at open() time.
   This allows the probe routine to use the old driver initialization
   interface. */

int ax88140_probe(struct device *dev)
{
	int cards_found =0;
	static int pci_index =0;	// Static, for multiple probe calls.

	/* Ideally we would detect all network cards in slot order. That would
	   be best done a central PCI probe dispatch, which wouldn't work
	   well with the current structure.  So instead we detect just in slot order. */

	if (pcibios_present())
		for (;pci_index < 0xff; pci_index++) {
			unsigned char pci_bus, pci_device_fn;
			unsigned char pci_irq_line, pci_latency;
			unsigned short pci_command, vendor, device;
			unsigned int pci_ioaddr, chip_idx =0;

			if (pcibios_find_class(PCI_CLASS_NETWORK_ETHERNET << 8,
				reverse_probe ? 0xfe -pci_index :pci_index,
				&pci_bus, &pci_device_fn) != PCIBIOS_SUCCESSFUL)
				if (reverse_probe)
					continue;
				else
					break;
			pcibios_read_config_word(pci_bus, pci_device_fn,
				PCI_VENDOR_ID, &vendor);
			pcibios_read_config_word(pci_bus, pci_device_fn,
				PCI_DEVICE_ID, &device);
			pcibios_read_config_byte(pci_bus, pci_device_fn,
				PCI_INTERRUPT_LINE, &pci_irq_line);
			pcibios_read_config_dword(pci_bus, pci_device_fn,
				PCI_BASE_ADDRESS_0, &pci_ioaddr);

			// Remove I/O space marker in bit 0.
			pci_ioaddr &= ~3;


			if (vendor != PCI_VENDOR_ID_ASIX)
				continue;

			for (chip_idx =0; ax88140_tbl[chip_idx].chip_name; chip_idx++)
				if (device == ax88140_tbl[chip_idx].device_id)
					break;
			if (ax88140_tbl[chip_idx].chip_name == 0) {
				printk(KERN_INFO"Unknown PCI ethernet chip type"
				" %4.4x"" detected: not configured.\n", device);
				continue;
			}
			#if AX88140_DEBUG > 2
			printk(KERN_DEBUG "Found AX88140 at I/O %#x, IRQ %d.\n",
				pci_ioaddr, pci_irq_line);
			#endif

			if (check_region(pci_ioaddr, AX88140_TOTAL_SIZE))
				continue;

#ifdef MODULE
			dev =ax88140_probe1(dev, pci_ioaddr, pci_irq_line, chip_idx,
						 cards_found);
#else
			dev =ax88140_probe1(dev, pci_ioaddr, pci_irq_line, chip_idx, -1);
#endif

			if (dev !=NULL) {
			  // Get and check the bus-master and latency values.
			  pcibios_read_config_word(pci_bus, pci_device_fn,
						PCI_COMMAND, &pci_command);
			  if (!(pci_command & PCI_COMMAND_MASTER)) {
				printk(KERN_INFO "  PCI Master Bit has not been set! Setting...\n");
				pci_command |= PCI_COMMAND_MASTER;
				pcibios_write_config_word(pci_bus, pci_device_fn,
						PCI_COMMAND, pci_command);
			  }
			  pcibios_read_config_byte(pci_bus, pci_device_fn,
						PCI_LATENCY_TIMER, &pci_latency);
			  if (pci_latency < 10) {
				printk(KERN_INFO "  PCI latency timer (CFLT) is unreasonably"
						" low at %d.  Setting to 64 clocks.\n", pci_latency);
				pcibios_write_config_byte(pci_bus, pci_device_fn,
						PCI_LATENCY_TIMER, 64);
			  } 
			  #if AX88140_DEBUG > 1
			  printk(KERN_INFO"  PCI latency timer (CFLT) is %#x.\n", pci_latency);
			  #endif
			  // Bring the 21143 out power-down mode.
//			  if (device == PCI_DEVICE_ID_DEC_AX88140_21142)
//				pcibios_write_config_dword(pci_bus, pci_device_fn, 0x40, 0x40000000);
			  dev =0;
			  cards_found++;
			}
		}
	return cards_found ? 0 :-ENODEV;
}


#ifdef MODULE
#if LINUX_VERSION_CODE > 0x20118
//MODULE_PARM(debug, "i");
//static int debug =-1;
MODULE_PARM(reverse_probe, "i");
MODULE_PARM(rx_copybreak, "i");
MODULE_PARM(options, "1-" __MODULE_STRING(8) "i");
MODULE_PARM(full_duplex, "1-" __MODULE_STRING(8) "i");
#endif

void
cleanup_module(void)
{
	struct device *next_dev;

	// No need to check MOD_IN_USE, as sys_delete_module() checks.
	while (root_ax88140_dev) {
		next_dev =((struct ax88140_private *)root_ax88140_dev->priv)->next_module;
		unregister_netdev(root_ax88140_dev);
		release_region(root_ax88140_dev->base_addr, AX88140_TOTAL_SIZE);
		kfree(root_ax88140_dev);
		root_ax88140_dev =next_dev;
	}
}

int
init_module(void)
{
	return ax88140_probe(root_ax88140_dev =NULL);
}


#endif	// MODULE 

/*
  local variables:
  compile-command: "gcc -DMODVERSIONS -DMODULE -D__KERNEL__ -DAX88140_DEBUG=0 -m486 -I/usr/src/linux/net/inet -Wall -Wredundant-decls -Wstrict-prototypes -O6 -c ax88140.c"
  c-indent-level: 4
  c-basic-offset: 4
  tab-width: 4
  End:
*/

